﻿--[[
    BankCalc see version in BankCalc.toc.
    author: stelised90
    email: stelised90@yahoo.com
    web: www.curseforge.com
]]

--Sudo General Namespaces and globals
BankCalc = {}
BankCalc.events = {}
BankCalc_Settings = { }
BankCalc.RemapBindings = { }

BankCalc.NumberDisplay = "0"
BankCalc.RunningTotal = ""
BankCalc.PreviousKeyType = "none"
BankCalc.PreviousOP = ""

BankCalc.ConsoleLastAns = "0"
BankCalc.MemoryIndicator = ""
BankCalc.MemoryIndicatorON = "M"
BankCalc.MemoryNumber = "0"
BankCalc.MemorySet = "0"

BankCalc.BindingMap = {
    NUMLOCK = "XC_NUMLOCK",
    HOME = "XC_CLEAR",
    BACKSPACE = "XC_BACKSPACE",
    NUMPADDIVIDE = "XC_DIV",
    NUMPADMULTIPLY = "XC_MUL",
    NUMPADMINUS = "XC_SUB",
    NUMPADPLUS = "XC_ADD",
    ENTER = "XC_EQ",
    NUMPAD0 = "XC_0",
    NUMPAD1 = "XC_1",
    NUMPAD2 = "XC_2",
    NUMPAD3 = "XC_3",
    NUMPAD4 = "XC_4",
    NUMPAD5 = "XC_5",
    NUMPAD6 = "XC_6",
    NUMPAD7 = "XC_7",
    NUMPAD8 = "XC_8",
    NUMPAD9 = "XC_9",
    NUMPADDECIMAL = "XC_DEC"
    }


--Register to addon load event
local frame = CreateFrame("Frame")

--Main Initialization
function BankCalc.events:ADDON_LOADED(arg1, ...)
	if( arg1 == "BankCalc") then
		--Mod Initialization
		SlashCmdList["BankCalc"] = BankCalc.cmdline
		SLASH_BankCalc1 = "/BankCalc"
		SLASH_BankCalc2 = "/calc"
		SLASH_BankCalc3 = "/="
	    BankCalc.optionvariables()
	    BankCalc.minimap_init()
	    BankCalc_VERSION = GetAddOnMetadata("BankCalc", "Version")
	end
end

frame:SetScript("OnEvent", function(self, event, ...) BankCalc.events[event](self, ...) end)
for k, v in pairs(BankCalc.events) do
	frame:RegisterEvent(k)
end


--Fuction for setting up Saved Variables
function BankCalc.optionvariables()
    if (BankCalc_Settings.Binding == nil) then
        BankCalc_Settings.Binding = 1
    end
    if (BankCalc_Settings.Minimapdisplay == nil) then
        BankCalc_Settings.Minimapdisplay = 1
    end
    if (BankCalc_Settings.Minimappos == nil) then
        BankCalc_Settings.Minimappos = 295
    end
end

--[[--------------------------------------------------------------------
    Function for adding Debug messages via BankCalc.debug("message") call
    
    --------------------------------------------------------------------]]
function BankCalc.debug(debugmsg)
    ChatFrame1:AddMessage("BankCalc debug: " .. debugmsg)
end

--Function for handling the chat slash commands
function BankCalc.cmdline(msg)
	-- this function handles our chat command
	if (msg == nil or msg == "") then
		BankCalc.windowdisplay()
		return nil
	end

	local expression = msg

	newexpression = BankCalc.parse(expression)

	local result = BankCalc.BankCalculate(newexpression)

	if ( result == nil ) then
		result = 'nil'
	end

	BankCalc.ConsoleLastAns = result

	--message(result)
	ChatFrame1:AddMessage("BankCalc Result: " .. expression .. " = " .. result, 1.0, 1.0, 0.5)
end

--Processes for binding and unbinding numberpad keys to BankCalc
function BankCalc.rebind()
    if (BankCalc_Settings.Binding == 1) then
    	for key,value in pairs(BankCalc.BindingMap) do
    		BankCalc.RemapBindings[key] = GetBindingAction(key)
    	end
        for key,value in pairs(BankCalc.BindingMap) do
        	SetBinding(key, value)
        end
    end
end

function BankCalc.unbind()
    if (BankCalc_Settings.Binding == 1) then
        for key,value in pairs(BankCalc.RemapBindings) do
        	SetBinding(key, value)
        end
    end
end

--Handle Key Inputs
function BankCalc.buttoninput(key)
	if ( key == "CL" ) then
		BankCalc.clear()
	elseif ( key == "CE") then
		BankCalc.ce()
	elseif ( key == "PM" ) then
		BankCalc.plusminus()
	elseif ( key == "GOLD" ) then
		BankCalc.stategold()
	elseif ( key == "SILVER" ) then
		BankCalc.statesilver()
	elseif ( key == "COPPER" ) then
		BankCalc.statecopper()
	elseif ( key == "MC" ) then
		BankCalc.mc()
	elseif ( key == "MA" ) then
		BankCalc.ma()
	elseif ( key == "MS" ) then
		BankCalc.ms()
	elseif ( key == "MR" ) then
		BankCalc.mr()
	elseif ( key == "BS" ) then
		BankCalc.backspace()
	elseif (key == "=" or key == "/" or key == "*" or key == "-" or key == "-" or key == "+" or key == "^") then
		BankCalc.funckey(key)
	else
		BankCalc.numkey(key)
	end
end

--Button Clear
function BankCalc.clear()
    BankCalc.RunningTotal = ""
    BankCalc.PreviousKeyType = "none"
    BankCalc.PreviousOP = ""
    BankCalc.display("0")
end

--Button CE
function BankCalc.ce()
    BankCalc.display("0")
end

--Button Backspace
function BankCalc.backspace()
    local currText = BankCalc.NumberDisplay
    if (currText == "0") then
        return
    else
        length = string.len(currText)-1
        if (length < 0) then
            length = 0
        end
        currText = string.sub(currText,0,length)
        if (string.len(currText) < 1) then
            BankCalc.display("0")
        else
            BankCalc.display(currText)
        end
    end
end

--Button Plus Minus Key
function BankCalc.plusminus()
    local currText = BankCalc.NumberDisplay
    if (currText ~= "0") then
		if (string.find(currText, "-")) then
            currText = string.sub(currText, 2)
		else
			currText = "-" .. currText
		end
	end
    BankCalc.PreviousKeyType = "state"
    BankCalc.display(currText)
end

--Button Gold (state)
function BankCalc.stategold()
    local currText = BankCalc.NumberDisplay
	if (string.find(currText, "[csg]") == nil) then
		currText = currText .. "g"
	end
    BankCalc.PreviousKeyType = "state"
    BankCalc.display(currText)
end

--Button Silver (state)
function BankCalc.statesilver()
    local currText = BankCalc.NumberDisplay
	if (string.find(currText, "[cs]") == nil) then
		currText = currText .. "s"
	end
    BankCalc.PreviousKeyType = "state"
    BankCalc.display(currText)
end

--Button Copper (state)
function BankCalc.statecopper()
    local currText = BankCalc.NumberDisplay
	if (string.find(currText, "c") == nil) then
		currText = currText .. "c"
	end
    BankCalc.PreviousKeyType = "state"
    BankCalc.display(currText)
end

--Button Memory Clear
function BankCalc.mc()
    BankCalc.MemoryNumber = "0"
    BankCalc.display(BankCalc.NumberDisplay, "0")
end

--Button Memory Add
function BankCalc.ma()
    temp = BankCalc.parse(BankCalc.MemoryNumber .. "+" .. BankCalc.NumberDisplay)
    BankCalc.MemoryNumber = BankCalc.BankCalculate(temp)
    BankCalc.display("0","1")
    BankCalc.clear()
end

--Button Memory Store
function BankCalc.ms()
    BankCalc.MemoryNumber = BankCalc.parse(BankCalc.NumberDisplay)
    BankCalc.display("0","1")
    BankCalc.clear()
end

--Button Memory Recall
function BankCalc.mr()
    BankCalc.display(BankCalc.MemoryNumber)
end

--Sets up the function keys ie, + - * / =
function BankCalc.funckey(key)
	local currText = BankCalc.NumberDisplay
    if ( IsShiftKeyDown() and key == "=" ) then
        ChatFrame_OpenChat("")
        return
    end
	if (BankCalc.PreviousKeyType=="none" or BankCalc.PreviousKeyType=="num" or BankCalc.PreviousKeyType=="state") then
			if (key == "/" or key == "*" or key == "-" or key == "-" or key == "+" or key == "^") then
					
				if (BankCalc.PreviousOP~="" and BankCalc.PreviousOP ~= "=") then
					temp = BankCalc.parse(BankCalc.RunningTotal .. BankCalc.PreviousOP .. currText)
					currText = BankCalc.BankCalculate(temp)
				end
				BankCalc.RunningTotal = currText
				BankCalc.PreviousOP = key
			elseif (key == "=") then
				if BankCalc.PreviousOP ~= "=" and  BankCalc.PreviousOP ~= "" then
					temp = BankCalc.parse(BankCalc.RunningTotal .. BankCalc.PreviousOP .. currText)
					currText = BankCalc.BankCalculate(temp)
					BankCalc.RunningTotal = currText
					BankCalc.PreviousOP="="
				end
			end
				
	else --must be a func key, a second+ time
		if (key == "/" or key == "*" or key == "-" or key == "-" or key == "+" or key == "^") then
			BankCalc.PreviousOP=key
		else
			BankCalc.PreviousOP=""
		end 
	end
	BankCalc.PreviousKeyType = "func"
	BankCalc.display(currText)
end

--Manage Number Inputs
function BankCalc.numkey(key)
	local currText = BankCalc.NumberDisplay
	
	if (BankCalc.PreviousKeyType=="none" or BankCalc.PreviousKeyType=="num" or BankCalc.PreviousKeyType=="state")then
		if (key == ".") then
			if (string.find(currText, "[csg%.]") == nil) then
				currText = currText .. "."
			end
		else
			if (currText == "0") then
				currText = ""
			end	

			currText = currText .. key
		end
	else
		if (key == ".") then
			currText = "0."
		else
			currText = key
		end
	end

	BankCalc.PreviousKeyType = "num"
    BankCalc.display(currText)
end

--Send the number display to an open chatbox
function BankCalc.numberdisplay_click(button, ignoreShift)
	if ( button == "LeftButton" ) then
		if ( IsShiftKeyDown() and not ignoreShift ) then
			if ( ChatFrameEditBox:IsVisible() ) then
				ChatFrameEditBox:Insert(BankCalc.NumberDisplay)
			end
		end
	end
end


--[[-----------------------------------------------------------------------------------
    Where the Calculations occur
    On a side note, Simple is easier, getting into complex if/then/elseif/else statements
    to perform math functions may introduce unexpected results... maybe.
    -----------------------------------------------------------------------------------]]
function BankCalc.BankCalculate(expression)
	local tempvar = "QCExpVal"

	setglobal(tempvar, nil)
	RunScript(tempvar .. "=(" .. expression .. ")")
	local result = getglobal(tempvar)

	return result
end

--This function parses the input for the money functions
function BankCalc.parse(expression)
	local ismoney = false

	newexpression = expression

	local newexpression = string.gsub(newexpression, "ans", BankCalc.ConsoleLastAns)

	-- g s c
	local newexpression = string.gsub(newexpression, "%d+g%d+s%d+c", function (a)
			ismoney = true
			return BankCalc.FromGSC(a)
		end )

	-- g s
	local newexpression = string.gsub(newexpression, "%d+g%d+s", function (a)
			ismoney = true
			return BankCalc.FromGSC(a)
		end )


	-- g   c
	local newexpression = string.gsub(newexpression, "%d+g%d+c", function (a)
			ismoney = true
			return BankCalc.FromGSC(a)
		end )

	-- g         allows #.#
	local newexpression = string.gsub(newexpression, "%d+%.?%d*g", function (a)
			ismoney = true
			return BankCalc.FromGSC(a)
		end )

	--   s c
	local newexpression = string.gsub(newexpression, "%d+s%d+c", function (a)
			ismoney = true
			return BankCalc.FromGSC(a)
		end )

	--   s       allows #.#
	local newexpression = string.gsub(newexpression, "%d+%.?%d*s", function (a)
			ismoney = true
			return BankCalc.FromGSC(a)
		end )

	--     c
	local newexpression = string.gsub(newexpression, "%d+c", function (a)
			ismoney = true
			return BankCalc.FromGSC(a)
		end )


	if (ismoney) then
		newexpression = "BankCalc.ToGSC(" .. newexpression .. ")"
	end

	return newexpression
end

--The following two functions do the to and from gold calculations
function BankCalc.ToGSC(decimal, std)
	local gold = 0
	local silver = 0
	local copper = 0

	if (std == "gold") then
		copper = math.fmod(decimal, .01)
		decimal = decimal - copper
		copper = copper * 10000

		silver = math.fmod(decimal, 1)
		decimal = decimal - silver
		silver = silver * 100

		gold = decimal
	elseif (std == "silver") then
		copper = math.fmod(decimal, 1)
		decimal = decimal - copper
		copper = copper * 100

		silver = math.fmod(decimal, 100)
		decimal = decimal - silver

		gold = decimal / 100
	else
		copper = math.fmod(decimal, 100)
		decimal = decimal - copper

		silver = math.fmod(decimal, 10000)
		decimal = decimal - silver
		silver = silver / 100

		gold = decimal / 10000
	end

	local temp = ""

	if (gold > 0) then
		temp = temp .. gold .. "g"
	end
	if (silver > 0 or (gold > 0 and copper > 0)) then
		temp = temp .. silver .. "s"
	end
	if (copper > 0) then
		temp = temp .. copper .. "c"
	end

	return temp
end

function BankCalc.FromGSC(gold, silver, copper)
	if (gold == nil) then
		return ""
	end

	local total = 0

	if (type(gold) == "string" and (not silver or type(silver) == "nil") and (not copper or type(copper) == "nil")) then
		local temp = gold
		
		golds,golde = string.find(temp, "%d*%.?%d*g")
		if (golds == nil) then
			gold = 0
		else
			gold = string.sub(temp, golds, golde - 1)
		end
	
		silvers,silvere = string.find(temp, "%d*%.?%d*s")
		if (silvers == nil) then
			silver = 0
		else
			silver = string.sub(temp, silvers, silvere - 1)
		end

		coppers,coppere = string.find(temp, "%d*c")
		if (coppers == nil) then
			copper = 0
		else
			copper = string.sub(temp, coppers, coppere - 1)
		end
	end

	total = total + copper
	total = total + (silver * 100)
	total = total + (gold * 10000)

	return "" .. total
end
