--[[
    This file contains all the GUI instructions.
    The idea of using lua to do all the drawing makes sense to me
    even if it is potentially slower than xml to do the same.  Figure
    that most people are not going to need the calculator to open and
    run the whatever speed increase is of using xml.  I haven't noticed
    much difference between loading it via lua vs other mods that use xml
    ]]


--Display Main calculator Window
function BankCalc.windowdisplay()
	if (BankCalc_window == nil) then
		BankCalc.windowframe()
		BankCalc_window:Show()
	elseif (BankCalc_window:IsVisible()) then
		BankCalc_window:Hide()
    else
        BankCalc_window:Show()
		BankCalc.clear()
	end
end

--Display options window
function BankCalc.optiondisplay()
	if (BankCalc_optionwindow == nil) then
		BankCalc.optionframe()
		BankCalc_optionwindow:Show()
	elseif (BankCalc_optionwindow:IsVisible()) then
        BankCalc_optionwindow:Hide()
    else
		BankCalc_optionwindow:Show()
	end
end


function BankCalc.display(displaynumber, memoryset)
    if ( displaynumber == nil or displaynumber == "" ) then
        displaynumber = "0"
    elseif ( memoryset == "1" ) then
        BankCalc_memorydisplay:SetText ( BankCalc.MemoryIndicatorON )
    elseif ( memoryset == "0" ) then
        BankCalc_memorydisplay:SetText( BankCalc.MemoryIndicator )
    end
    BankCalc.NumberDisplay = displaynumber
    BankCalc_numberdisplay:SetText( displaynumber )
end

function BankCalc.minimap_init()
    if (BankCalc_Settings.Minimapdisplay == 1) then
        local frame = CreateFrame("Button","BankCalc_minimap_button",Minimap)
        frame:SetWidth(34)
        frame:SetHeight(34)
        frame:SetFrameStrata("LOW")
        frame:SetToplevel(1)
        frame:SetNormalTexture("Interface\\AddOns\\BankCalc\\BankCalc_ButtonRoundNormal.tga")
        frame:SetPushedTexture("Interface\\AddOns\\BankCalc\\BankCalc_ButtonRoundPushed.tga")
        frame:SetHighlightTexture("Interface/Minimap/UI-Minimap-ZoomButton-Highlight")
        frame:RegisterForClicks("AnyUp")
        frame:SetScript("OnClick", function(self, button, down) 
        	if (button == "LeftButton") then
        		BankCalc.windowdisplay()
        	elseif (button == "RightButton") then
        		BankCalc.optiondisplay()
        	end
        end)
        frame:SetScript("OnEnter", function() BankCalc.tooltip("minimap") end)
        frame:SetScript("OnLeave", function() BankCalc.tooltip("hide") end)
        BankCalc.minimapbutton_updateposition()
        frame:Show()
    end
end

--Minimap button Position
function BankCalc.minimapbutton_updateposition()

   BankCalc_minimap_button:SetPoint("TOPLEFT", "Minimap", "TOPLEFT",
         54 - (78 * cos(BankCalc_Settings.Minimappos)),
         (78 * sin(BankCalc_Settings.Minimappos)) - 55)

end


--Tooltip display
function BankCalc.tooltip(mouseover)
    if ( mouseover == "minimap" ) then
        GameTooltip:SetOwner(BankCalc_minimap_button , "ANCHOR_BOTTOMLEFT")
        GameTooltip:SetText("Show/Hide BankCalc")
    else
        GameTooltip : Hide ()
    end
end

--Function for handeling Binding checkbox
function BankCalc.options_binding()
    if (BankCalc_options_bindcheckbox:GetChecked() == 1) then
        BankCalc_Settings.Binding = 1
    else
        BankCalc.unbind()
        BankCalc_Settings.Binding = 0
    end
end

-- Function for Handeling Minimap Display checkbox
function BankCalc.options_minimapdisplay()
    if (BankCalc_options_minimapcheckbox:GetChecked() == 1) then
        BankCalc_Settings.Minimapdisplay = 1
        if (BankCalc_minimap_button == nil) then
            BankCalc.minimap_init()
        else
            BankCalc_minimap_button:Show()
        end
    else
        BankCalc_Settings.Minimapdisplay = 0
        BankCalc_minimap_button:Hide()
    end
end

-- Function for managing options slider
function BankCalc.options_minimapslidercontrol()
	if (BankCalc_Settings.Minimapdisplay == 1) then
		BankCalc_Settings.Minimappos = BankCalc_options_minimapslider:GetValue()
		BankCalc.minimapbutton_updateposition()
		else
			BankCalc_options_minimapslider:SetValue(BankCalc_Settings.Minimappos)
			return
	end
end

--Draw the main window
function BankCalc.windowframe()
    --Main Window Frame (container) and title bar
    local frame = CreateFrame("Frame","BankCalc_window",UIParent)
    frame:SetFrameStrata("HIGH")
    frame:EnableMouse(true)
    frame:EnableKeyboard(true)
    frame:SetMovable(true)
    frame:SetHeight(307)
    frame:SetWidth(240)
    frame:SetScript("OnMouseDown", function() frame:StartMoving() end)
    frame:SetScript("OnMouseUp", function() frame:StopMovingOrSizing() end)
    frame:SetScript("OnShow", function() BankCalc.rebind() end)
    frame:SetScript("OnHide", function() BankCalc.unbind() end)
    frame:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background",
                        edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
                        tile = true, tileSize = 32, edgeSize = 32,
                        insets = { left = 11, right = 12, top = 12, bottom = 11 }})
    frame:SetPoint("CENTER",0,0)
    local titletexture = frame:CreateTexture("BankCalc_window_titletexture")
    titletexture:SetHeight(32)
    titletexture:SetWidth(160)
    titletexture:SetTexture("Interface/DialogFrame/UI-DialogBox-Header")
    titletexture:SetTexCoord(0.2, 0.8, 0, 0.6)
    titletexture:SetPoint("TOP",0,5)
    local titlefont = frame:CreateFontString("BankCalc_windowtest_titlefont")
    titlefont:SetHeight(0)
    titlefont:SetWidth(140)
    titlefont:SetFont("Fonts/FRIZQT__.TTF",12)
    titlefont:SetPoint("TOP",0,-4)
    titlefont:SetTextColor(1,0.8196079,0)
    titlefont:SetText("BankCalc v" .. BankCalc_VERSION)
    --Number Display box
    local numberdisplaybackground = frame:CreateTexture("BankCalc_numberdisplaybackground")
    numberdisplaybackground:SetHeight(34)
    numberdisplaybackground:SetWidth(215)
    numberdisplaybackground:SetTexture("interface/chatframe/ui-chatinputborder")
    numberdisplaybackground:SetPoint("TOPLEFT",10,-33)
    local numberdisplay = frame:CreateFontString("BankCalc_numberdisplay","GameFontWhite")
    numberdisplay:SetHeight(34)
    numberdisplay:SetWidth(205)
    numberdisplay:SetFont("Fonts/FRIZQT__.TTF",12)
    numberdisplay:SetJustifyH("RIGHT")
    numberdisplay:SetPoint("TOPLEFT",10,-33)
    numberdisplay:SetText(BankCalc.NumberDisplay)
    --Memory Display
    local memorydisplay = frame:CreateFontString("BankCalc_memorydisplay","GameFontNormal")
    memorydisplay:SetWidth(29)
    memorydisplay:SetHeight(29)
    memorydisplay:SetFont("Fonts/FRIZQT__.TTF",12)
    memorydisplay:SetPoint("TOPLEFT",15,-73)
    --memorydisplay:SetText("M")
    --ExitButton
    local exitbutton = CreateFrame("Button", "BankCalc_exitbutton",frame,"UIPanelCloseButton")
    exitbutton:SetPoint("TOPRIGHT",-4,-4)
	exitbutton:SetScript("OnClick", function() BankCalc.windowdisplay() end)

	--Main calculator buttons
    BankCalc.button(75,29,50,-73,"Backspace","BS")
    BankCalc.button(41,29,131,-73,"CE","CE")
    BankCalc.button(41,29,178,-73,"C","CL")
    BankCalc.button(29,70,190,-183,"=","=")
    BankCalc.button(29,32,190,-146,"^","^")
    BankCalc.button(29,32,190,-108,"+/-","PM")
    BankCalc.button(29,32,155,-221,"+","+")
    BankCalc.button(29,32,155,-183,"-","-")
    BankCalc.button(29,32,155,-145,"*","*")
    BankCalc.button(29,32,155,-108,"/","/")
    BankCalc.button(29,32,120,-259,"c","COPPER")
    BankCalc.button(29,32,85,-259,"s","SILVER")
    BankCalc.button(29,32,50,-259,"g","GOLD")
    BankCalc.button(29,32,120,-221,".",".")
    BankCalc.button(64,32,50,-222,"0","0")
    BankCalc.button(29,32,50,-184,"1","1")
    BankCalc.button(29,32,85,-183,"2","2")
    BankCalc.button(29,32,120,-183,"3","3")
    BankCalc.button(29,32,50,-146,"4","4")
    BankCalc.button(29,32,85,-146,"5","5")
    BankCalc.button(29,32,120,-146,"6","6")
    BankCalc.button(29,32,50,-108,"7","7")
    BankCalc.button(29,32,85,-108,"8","8")
    BankCalc.button(29,32,120,-108,"9","9")
    BankCalc.button(29,32,15,-221,"MA","MA")
    BankCalc.button(29,32,15,-183,"MS","MS")
	BankCalc.button(29,32,15,-146,"MR","MR")
    BankCalc.button(29,32,15,-108,"MC","MC")
    
    --Option show button
    local optionbutton = CreateFrame("Button", "BankCalc_optionwindow_button",frame,"UIPanelButtonTemplate")
    optionbutton:SetWidth(70)
    optionbutton:SetHeight(25)
    optionbutton:SetPoint("BOTTOMRIGHT",-15,15)
    optionbutton:SetText("Options")
    optionbutton:SetScript("OnClick", function() BankCalc.optiondisplay() end)
    BankCalc.rebind()
    tinsert(UISpecialFrames,"BankCalc_window")
end

function BankCalc.button(width, height, x, y, text, cmd)
	local button = CreateFrame("Button", "BankCalc." .. text, BankCalc_window ,"UIPanelButtonTemplate")
	button:SetWidth(width)
	button:SetHeight(height)
	button:SetPoint("TOPLEFT",x,y)
	button:SetText(text)
	button:SetScript("OnClick", function() BankCalc.buttoninput(cmd) end)
	
end

--Draw the Option window
function BankCalc.optionframe()
    --Options window Frame
    local frame = CreateFrame("Frame","BankCalc_optionwindow",UIParent)
    frame:SetFrameStrata("HIGH")
    frame:EnableMouse(true)
    frame:SetWidth(220)
    frame:SetHeight(200)
    frame:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background",
                        edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
                        tile = true, tileSize = 32, edgeSize = 32,
                        insets = { left = 11, right = 12, top = 12, bottom = 11 }})
    frame:SetPoint("CENTER",230,0)
    local titletexture = frame:CreateTexture("BankCalc_optionwindow_titletexture")
    titletexture:SetHeight(32)
    titletexture:SetWidth(160)
    titletexture:SetTexture("Interface/DialogFrame/UI-DialogBox-Header")
    titletexture:SetTexCoord(0.2, 0.8, 0, 0.6)
    titletexture:SetPoint("TOP",0,5)
    local titlefont = frame:CreateFontString("BankCalc_optionwindow_titlefont")
    titlefont:SetHeight(0)
    titlefont:SetWidth(140)
    titlefont:SetFont("Fonts/FRIZQT__.TTF",12)
    titlefont:SetPoint("TOP",0,-4)
    titlefont:SetTextColor(1,0.8196079,0)
    titlefont:SetText("BankCalc Options")
    --Options Okay Button
    local okaybutton = CreateFrame("Button", "BankCalc_optionokaybutton",frame,"UIPanelButtonTemplate")
    okaybutton:SetWidth(70)
    okaybutton:SetHeight(29)
    okaybutton:SetPoint("BOTTOM",0,20)
    okaybutton:SetText("Okay")
    okaybutton:SetScript("OnClick", function() BankCalc.optiondisplay() end)
    --Binding Check box
    local bindingcheckbox = CreateFrame("CheckButton","BankCalc_options_bindcheckbox",frame,"OptionsCheckButtonTemplate")
    bindingcheckbox:SetPoint("TOPLEFT",15,-40)
    bindingcheckbox:SetChecked(BankCalc_Settings.Binding)
    bindingcheckbox:SetScript("OnClick", function() BankCalc.options_binding() end)
    local bindingcheckboxtext = frame:CreateFontString("BankCalc_options_bindcheckboxtext")
    bindingcheckboxtext:SetWidth(200)
    bindingcheckboxtext:SetHeight(0)
    bindingcheckboxtext:SetFont("Fonts/FRIZQT__.TTF",10)
    bindingcheckboxtext:SetTextColor(1,0.8196079,0)
    bindingcheckboxtext:SetJustifyH("LEFT")
    bindingcheckboxtext:SetText("Use Automatic Key Bindings")
    bindingcheckboxtext:SetPoint("LEFT","BankCalc_options_bindcheckbox",30,0)
    --Display Minimap Check Box
    local minimapcheckbox = CreateFrame("CheckButton","BankCalc_options_minimapcheckbox",frame,"OptionsCheckButtonTemplate")
    minimapcheckbox:SetPoint("TOPLEFT",15,-70)
    minimapcheckbox:SetChecked(BankCalc_Settings.Minimapdisplay)
    minimapcheckbox:SetScript("OnClick", function() BankCalc.options_minimapdisplay() end)
    local minimapcheckboxtext = minimapcheckbox:CreateFontString("BankCalc_options_minimapcheckboxtext")
    minimapcheckboxtext:SetWidth(200)
    minimapcheckboxtext:SetHeight(0)
    minimapcheckboxtext:SetFont("Fonts/FRIZQT__.TTF",10)
    minimapcheckboxtext:SetTextColor(1,0.8196079,0)
    minimapcheckboxtext:SetJustifyH("LEFT")
    minimapcheckboxtext:SetText("Display Minimap Icon")
    minimapcheckboxtext:SetPoint("LEFT","BankCalc_options_minimapcheckbox",30,0)
    --Minimap Position Slider
    local minimapslider = CreateFrame("Slider","BankCalc_options_minimapslider",frame,"OptionsSliderTemplate")
    minimapslider:SetWidth(180)
    minimapslider:SetHeight(16)
    minimapslider:SetMinMaxValues(0, 360)
    minimapslider:SetValueStep(1)
	minimapslider:SetScript("OnValueChanged", function() BankCalc.options_minimapslidercontrol() end)
	BankCalc_options_minimapsliderHigh:SetText()
    BankCalc_options_minimapsliderLow:SetText()
    BankCalc_options_minimapsliderText:SetText("Minimap Button Position")
    minimapslider:SetPoint("TOPLEFT",15,-120)
	minimapslider:SetValue(BankCalc_Settings.Minimappos)

end

